<?php

namespace Drupal\firebase_ui\Controller;

use Symfony\Component\HttpFoundation\JsonResponse;
use Symfony\Component\HttpFoundation\Request;
use Drupal\Core\Controller\ControllerBase;
use Drupal\Core\Session\AccountProxyInterface;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Controller to handle Firebase UI token saving.
 */
class FirebaseUiController extends ControllerBase {

  /**
   * The current user.
   *
   * @var \Drupal\Core\Session\AccountProxyInterface
   */
  protected $currentUser;

  /**
   * The entity type manager.
   *
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   */
  protected $entityTypeManager;

  /**
   * Constructs a FirebaseUiController object.
   *
   * @param \Drupal\Core\Session\AccountProxyInterface $current_user
   *   The current user.
   * @param \Drupal\Core\Entity\EntityTypeManagerInterface $entity_type_manager
   *   The entity type manager.
   */
  public function __construct(AccountProxyInterface $current_user, EntityTypeManagerInterface $entity_type_manager) {
    $this->currentUser = $current_user;
    $this->entityTypeManager = $entity_type_manager;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container): self {
    return new static(
      $container->get('current_user'),
      $container->get('entity_type.manager')
    );
  }

  /**
   * Saves a Firebase token for the authenticated user.
   *
   * @param \Symfony\Component\HttpFoundation\Request $request
   *   The request object.
   *
   * @return \Symfony\Component\HttpFoundation\JsonResponse
   *   The JSON response indicating success or failure.
   */
  public function saveToken(Request $request): JsonResponse {
    $data = json_decode($request->getContent(), TRUE);
    $token = $data['token'] ?? NULL;

    if (!$this->currentUser->isAuthenticated() || !$token) {
      return new JsonResponse(['status' => 'error'], 403);
    }

    /** @var \Drupal\user\UserInterface $account */
    $account = $this->entityTypeManager->getStorage('user')->load($this->currentUser->id());

    $tokens = $account->get('field_firebase_tokens')->getValue();
    $existing_tokens = array_column($tokens, 'value');

    if (!in_array($token, $existing_tokens, TRUE)) {
      $tokens[] = ['value' => $token];
      $account->set('field_firebase_tokens', $tokens);
      $account->save();
    }

    return new JsonResponse(['status' => 'success']);
  }

}
