<?php

namespace Drupal\firebase_ui\Controller;

use Drupal\Component\Datetime\Time;
use Drupal\Core\Controller\ControllerBase;
use Drupal\Core\Queue\QueueFactory;
use Symfony\Component\HttpFoundation\RedirectResponse;
use Symfony\Component\HttpKernel\Exception\NotFoundHttpException;
use Drupal\firebase_ui\Entity\FirebaseNotification;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Handles Firebase Notification actions like pause/resume and resend.
 */
class NotificationActionController extends ControllerBase {

  /**
   * The time service.
   *
   * @var \Drupal\Component\Datetime\Time
   */
  protected $time;

  /**
   * The queue factory.
   *
   * @var \Drupal\Core\Queue\QueueFactory
   */
  protected $queue;

  /**
   * Constructs the NotificationActionController.
   *
   * @param \Drupal\Component\Datetime\Time $time
   *   The time service.
   * @param \Drupal\Core\Queue\QueueFactory $queue
   *   The queue factory.
   */
  public function __construct(Time $time, QueueFactory $queue) {
    $this->time = $time;
    $this->queue = $queue;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container): self {
    return new static(
      $container->get('datetime.time'),
      $container->get('queue')
    );
  }

  /**
   * Toggle status: Pause or Resume.
   *
   * @param \Drupal\firebase_ui\Entity\FirebaseNotification $firebase_notification
   *   The notification entity.
   *
   * @return \Symfony\Component\HttpFoundation\RedirectResponse
   *   A redirect response.
   */
  public function toggleStatus(FirebaseNotification $firebase_notification) {
    if (!$firebase_notification) {
      throw new NotFoundHttpException();
    }

    $current_status = $firebase_notification->get('status')->value;
    $firebase_notification->set('status', !$current_status);
    $firebase_notification->save();

    $message = $current_status
      ? $this->t('Notification "@title" has been paused.', ['@title' => $firebase_notification->get('title')->value])
      : $this->t('Notification "@title" has been resumed.', ['@title' => $firebase_notification->get('title')->value]);

    $this->messenger()->addStatus($message);
    return new RedirectResponse('/admin/config/services/firebase-ui/list');
  }

  /**
   * Resend: Duplicate the notification and queue it again.
   *
   * @param \Drupal\firebase_ui\Entity\FirebaseNotification $firebase_notification
   *   The notification entity.
   *
   * @return \Symfony\Component\HttpFoundation\RedirectResponse
   *   A redirect response.
   */
  public function resend(FirebaseNotification $firebase_notification) {
    if (!$firebase_notification) {
      throw new NotFoundHttpException();
    }

    // Clone the original notification.
    $new = FirebaseNotification::create([
      'type' => $firebase_notification->get('type')->value,
      'target_ids' => $firebase_notification->get('target_ids')->value,
      'title' => $firebase_notification->get('title')->value,
      'message' => $firebase_notification->get('message')->value,
      'send_time' => $this->time->getCurrentTime(),
      'status' => TRUE,
    ]);
    $new->save();

    // Add to queue.
    $this->queue->get('firebase_ui_notification_queue')
      ->createItem(['nid' => $new->id()]);

    $this->messenger()->addStatus($this->t('Notification "@title" has been duplicated and re-queued.', [
      '@title' => $firebase_notification->get('title')->value,
    ]));

    return new RedirectResponse('/admin/config/services/firebase-ui/list');
  }

}
